require 'json'

# 元データ（Classes.json）固定
INPUT_FILE  = "Classes.json"
# バックアップファイル拡張子
BK_EX_CHILD = ".bak"

# Lv上限
MAX_LEVEL = 999

# ◆能力値算出方式
#   0..データベース通り。
#      lv:現レベル  p[x]:レベル x の能力値
#      として、
#      (p[99] - p[98]) * (lv - 99)
#      をレベル 99 の能力値に加算。
LB_CALC_TYPE_DATABASE    = 0
#   1..データベースを用いた二次関数。
#        a:レベル1の値  b:レベル2の値  c:レベル3の値  x:現レベル
#      として、
#        ax^2 + bx + c
#      を用いて能力値を算出。
LB_CALC_TYPE_DB_2ND_FUNC = 1
#   2..データベースを用いた一次関数。
#        b:レベル2の値  c:レベル3の値  x:現レベル
#      として、
#        bx + c
#      を用いて能力値を算出。
#      (レベル2,3の値を使うのは、二次関数方式との使い分けを楽にするため)
LB_CALC_TYPE_DB_1ST_FUNC = 2
#   3..乱数方式。
#        レベル1の値をベースに
#        レベルアップする度に乱数で数値を加算し続ける方式
LB_CALC_TYPE_RANDOM      = 3
#   4..乱高下方式。
#        レベルが上がる毎に能力が乱高下する方式。
#        バカゲーとかの使用にどうぞ
LB_CALC_TYPE_BAKA        = 4

# 乱数方式で使用するシード値
#  HPのシード値
LB_CALC_BASE_HP  = 100
#  MPのシード値
LB_CALC_BASE_MP  = 50
#  HP/MP以外のステータスのシード値
LB_CALC_BASE_STS = 10

LB_CALC_FLG_HP  = "HP"
LB_CALC_FLG_MP  = "MP"
LB_CALC_FLG_STS = "STS"

# パラメータ生成関数
def calc_on_database(list, type)
  case type
  when LB_CALC_TYPE_DATABASE
    for lv in 100..MAX_LEVEL
      tmp = (list[99] - list[98]) * (lv - 99) + list[99]
    
      list.push(tmp)
    end
    return list
  when LB_CALC_TYPE_DB_2ND_FUNC
      for lv in 100..MAX_LEVEL
      tmp = (list[1] * lv)^2 + list[2] * lv + list[3]
    
      list.push(tmp)
    end
    return list
  when LB_CALC_TYPE_DB_1ST_FUNC
    for lv in 100..MAX_LEVEL
      tmp list[2] * lv + list[3]
    
      list.push(tmp)
    end
    return list
  when LB_CALC_TYPE_RANDOM
    tmp_list = [list[0], list[1]]
    for lv in 2..MAX_LEVEL
      case flg
      when LB_CALC_FLG_HP
        tmp = tmp_list[lv - 1] + rand(LB_CALC_BASE_HP)
      when LB_CALC_FLG_MP
        tmp = tmp_list[lv - 1] + rand(LB_CALC_BASE_MP)
      when LB_CALC_FLG_STS
        tmp = tmp_list[lv - 1] + rand(LB_CALC_BASE_STS)
      else
        p "異常なパラメータ種別フラグ"
        tmp = 0
      end
      
      tmp_list.push(tmp)
    end
    return tmp_list
  when LB_CALC_TYPE_BAKA
    tmp_list = [list[0], list[1]]
    for lv in 2..MAX_LEVEL
      case flg
      when LB_CALC_FLG_HP
        tmp = tmp_list[lv - 1] + rand(LB_CALC_BASE_HP * 2) - LB_CALC_BASE_HP
        tmp = [tmp, list[0]].max
      when LB_CALC_FLG_MP
        tmp = tmp_list[lv - 1] + rand(LB_CALC_BASE_MP * 2) - LB_CALC_BASE_MP
        tmp = [tmp, list[0]].max
      when LB_CALC_FLG_STS
        tmp = tmp_list[lv - 1] + rand(LB_CALC_BASE_STS * 2) - LB_CALC_BASE_STS
        tmp = [tmp, list[0]].max
      else
        p "異常なパラメータ種別フラグ"
        tmp = 0
      end
      
      tmp_list.push(tmp)
    end
    return tmp_list
  else
    p "異常なタイプ"
    return []
  end
end

def main
  mv_param_hash_list = []

  # ファイルが存在しない場合は終了
  if !FileTest.exist?(INPUT_FILE)
    p "Input File No Exist."
    exit 1
  end

  # ファイルロード JSON to Ruby(ハッシュ)
  File.open(INPUT_FILE) do |file|
    mv_param_hash_list = JSON.load(file)
    #p mv_param_hash_list[1]["params"]
  end

  tmp_mv_param_hash_list = mv_param_hash_list.dup

  #p "Before"
  #p tmp_mv_param_hash_list[1]["params"]
  #p tmp_mv_param_hash_list[1]["params"][0].size
  
  # Lv100以降のパラメータ生成
  for index in 1...tmp_mv_param_hash_list.size
    hp_list   = tmp_mv_param_hash_list[index]["params"][0]#.dup
    mp_list   = tmp_mv_param_hash_list[index]["params"][1]#.dup
    atk_list  = tmp_mv_param_hash_list[index]["params"][2]#.dup
    pdef_list = tmp_mv_param_hash_list[index]["params"][3]#.dup
    mat_list  = tmp_mv_param_hash_list[index]["params"][4]#.dup
    mdef_list = tmp_mv_param_hash_list[index]["params"][5]#.dup
    agi_list  = tmp_mv_param_hash_list[index]["params"][6]#.dup
    luk_list  = tmp_mv_param_hash_list[index]["params"][7]#.dup
    
    if (hp_list.size == 100)
      hp_list2   = calc_on_database(hp_list,   LB_CALC_TYPE_DATABASE)
    else
      p "Input File on Param in HP is already Generated!!"
    end
    if (mp_list.size == 100)
      mp_list2   = calc_on_database(mp_list,   LB_CALC_TYPE_DATABASE)
    else
      p "Input File on Param in MP is already Generated!!"
    end
    if (atk_list.size == 100)
      atk_list2  = calc_on_database(atk_list,  LB_CALC_TYPE_DATABASE)
    else
      p "Input File on Param in atk is already Generated!!"
    end
    if (pdef_list.size == 100)
      pdef_list2 = calc_on_database(pdef_list, LB_CALC_TYPE_DATABASE)
    else
      p "Input File on Param in pdef is already Generated!!"
    end
    if (mat_list.size == 100)
      mat_list2  = calc_on_database(mat_list,  LB_CALC_TYPE_DATABASE)
    else
      p "Input File on Param in mat is already Generated!!"
    end
    if (mdef_list.size == 100)
      mdef_list2 = calc_on_database(mdef_list, LB_CALC_TYPE_DATABASE)
    else
      p "Input File on Param in mdef is already Generated!!"
    end
    if (agi_list.size == 100)
      agi_list2  = calc_on_database(agi_list,  LB_CALC_TYPE_DATABASE)
    else
      p "Input File on Param in agi is already Generated!!"
    end
    if (luk_list.size == 100)
      luk_list2  = calc_on_database(luk_list,  LB_CALC_TYPE_DATABASE)
    else
      p "Input File on Param in luk is already Generated!!"
    end
  end

  #p "After"
  #p tmp_mv_param_hash_list[1]["params"]

  # Ruby(ハッシュ)からJSON形式への変換
  param_list_json = JSON.pretty_generate(tmp_mv_param_hash_list)

  # 旧ファイルのリネーム
  File.rename(INPUT_FILE, INPUT_FILE + BK_EX_CHILD)

  # ファイル出力
  file = File.open(INPUT_FILE, "w")
  file.write(param_list_json)
  file.close

  #File.open(OUTPUT_FILE, 'w') do |file|
  #  str = JSON.dump(tmp_mv_param_hash_list, file)
  #end

  p "Generate Complete!!"
end

# メイン関数の呼び出し
main

